unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Buttons, Math, StdCtrls, Menus, IniFiles, Clipbrd,
  HotKeySpy, ImgList;

type
  TColorDispFormat = (cdfRGBHex, cdfRGBDec, cdfDelphi, cdfVC, cdfVB, cdfHtml);

const
  DefZoomRate = 15;
  DefColorDispFormat = cdfRGBDec;

type
  TMainForm = class(TForm)
    Panel1: TPanel;
    MagnifierPanel: TPanel;
    ColorSamplePanel: TPanel;
    Panel4: TPanel;
    MousePosPanel: TPanel;
    ColorValuePanel: TPanel;
    MenuPanel: TPanel;
    ZoomRate2Button: TSpeedButton;
    ZoomRate5Button: TSpeedButton;
    ZoomRate10Button: TSpeedButton;
    ZoomRate15Button: TSpeedButton;
    ZoomRate30Button: TSpeedButton;
    MagnifierImage: TImage;
    MousePosEdit: TEdit;
    ColorValueEdit: TEdit;
    ColorFmtArrowButton: TSpeedButton;
    ColorFmtPopupMenu: TPopupMenu;
    ColorFmtPopRGBHexItem: TMenuItem;
    ColorFmtPopRGBDecItem: TMenuItem;
    ColorFmtPopDelphiItem: TMenuItem;
    ColorFmtPopVCItem: TMenuItem;
    ColorFmtPopVBItem: TMenuItem;
    ColorFmtPopHtmlItem: TMenuItem;
    HotKeySpy: THotKeySpy;
    CaptureTimer: TTimer;
    ZoomRate1Button: TSpeedButton;
    MainPopupMenu: TPopupMenu;
    MainPopExitItem: TMenuItem;
    MainPopN3Item: TMenuItem;
    MainPopHelpItem: TMenuItem;
    MainPopStartCapItem: TMenuItem;
    MainPopStopCapItem: TMenuItem;
    MainPopCopyImageItem: TMenuItem;
    MainPopN2Item: TMenuItem;
    MainPopN1Item: TMenuItem;
    MainPopCopyColorItem: TMenuItem;
    TipPaintBox: TPaintBox;
    ImageList: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MagnifierImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MagnifierImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure MagnifierImageMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure ColorFmtArrowButtonClick(Sender: TObject);
    procedure MenuPanelMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ColorFmtPopItemClick(Sender: TObject);
    procedure HotKeySpyHotKeys0HotKey(Sender: TObject);
    procedure HotKeySpyHotKeys1HotKey(Sender: TObject);
    procedure CaptureTimerTimer(Sender: TObject);
    procedure ColorValueEditClick(Sender: TObject);
    procedure ZoomRateButtonClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure HotKeySpyHotKeys2HotKey(Sender: TObject);
    procedure MainPopStartCapItemClick(Sender: TObject);
    procedure MainPopStopCapItemClick(Sender: TObject);
    procedure MainPopCopyImageItemClick(Sender: TObject);
    procedure MainPopCopyColorItemClick(Sender: TObject);
    procedure MainPopHelpItemClick(Sender: TObject);
    procedure MainPopExitItemClick(Sender: TObject);
    procedure TipPaintBoxPaint(Sender: TObject);
    procedure TipPaintBoxMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure MainPopupMenuPopup(Sender: TObject);
    procedure ColorValueEditDblClick(Sender: TObject);
  private
    FMagnifierDraging: Boolean;
    FHotKeyCapturing: Boolean;
    FZoomRate: Integer;
    FCurScreenPos: TPoint;
    FCurColor: TColor;
    FCurBitmap: TBitmap;
    FColorDispFormat: TColorDispFormat;
    FColorPicked: Boolean;

    procedure InitDefaultOptions;
    procedure LoadOptions;
    procedure SaveOptions;
    function GetIniFileName: string;

    procedure InitUI;
    procedure Reset;
    procedure DrawInactiveMagnifier;
    procedure DrawColorSampleBox;
    procedure ShowMousePos;
    procedure ShowColorValue;
    procedure ShowTipBox(Value: Boolean);

    function IsCapturing: Boolean;
    function IsInMagnifier(const Pos: TPoint): Boolean;
    procedure Capture(const ScreenPos: TPoint); 
    procedure MoveByKeyboard(Key: Word; Shift: TShiftState);
    procedure CopyColorValue;
    procedure CopyBitmap;
  end;

var
  MainForm: TMainForm;

implementation

uses PubUtils;

{$R *.dfm}

{ Misc }

//-----------------------------------------------------------------------------
// : RGBTColor
//-----------------------------------------------------------------------------
function GetColorFromRGB(R, G, B: Integer): TColor;
begin
  R := EnsureRange(R, 0, 255);
  G := EnsureRange(G, 0, 255);
  B := EnsureRange(B, 0, 255);

  Result := TColor(
    ((B and $FF) shl 16) or
    ((G and $FF) shl 8) or
    ((R and $FF) shl 0)
    );
end;

//-----------------------------------------------------------------------------
// : TColorRGB
//-----------------------------------------------------------------------------
procedure GetRGBFromColor(Color: TColor; var R, G, B: Integer);
begin
  R := Color and $000000FF;
  G := (Color and $0000FF00) shr 8;
  B := (Color and $00FF0000) shr 16;
end;

{ TMainForm }

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FCurBitmap := TBitmap.Create;

  Application.HintPause := 300;
  Left := Screen.WorkAreaRect.Right - Width;
  Top := Screen.WorkAreaRect.Bottom - Height;

  InitDefaultOptions;
  LoadOptions;
  InitUI;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SaveOptions;
  FCurBitmap.Free;
end;

procedure TMainForm.InitDefaultOptions;
begin
  FZoomRate := DefZoomRate;
  FColorDispFormat := DefColorDispFormat;
end;

procedure TMainForm.LoadOptions;
var
  Ini: TIniFile;
  Section: string;
begin
  Ini := TIniFile.Create(GetIniFileName);
  try
    Section := 'Options';
    FZoomRate := Ini.ReadInteger(Section, 'ZoomRate', FZoomRate);
    FZoomRate := TrimValue(FZoomRate, 1, 30);

    FColorDispFormat := TColorDispFormat(Ini.ReadInteger(Section, 'ColorDispFormat', Integer(FColorDispFormat)));
    FColorDispFormat := TColorDispFormat(TrimValue(Integer(FColorDispFormat), Ord(Low(TColorDispFormat)), Ord(High(TColorDispFormat))));
  finally
    Ini.Free;
  end;
end;

procedure TMainForm.SaveOptions;
var
  Ini: TIniFile;
  Section: string;
begin
  Ini := TIniFile.Create(GetIniFileName);
  try
    Section := 'Options';
    Ini.WriteInteger(Section, 'ZoomRate', FZoomRate);
    Ini.WriteInteger(Section, 'ColorDispFormat', Integer(FColorDispFormat));
  finally
    Ini.Free;
  end;
end;

function TMainForm.GetIniFileName: string;
begin
  Result := GetAppSubPath + ExtractFileMainName(Application.ExeName) + '.ini';
end;

procedure TMainForm.InitUI;
var
  Button: TSpeedButton;
begin
  Reset;
  ColorFmtPopupMenu.Items.Items[Integer(FColorDispFormat)].Checked := True;
  Button := TSpeedButton(FindComponent(Format('ZoomRate%dButton', [FZoomRate])));
  if Button <> nil then
    Button.Down := True;
end;

procedure TMainForm.Reset;
begin
  FCurColor := clBlack;
  FColorPicked := False;
  FCurScreenPos := Mouse.CursorPos;
  FCurBitmap.Assign(nil);

  DrawInactiveMagnifier;
  DrawColorSampleBox;
  ShowMousePos;
  ShowColorValue;
  ShowTipBox(True);
end;

procedure TMainForm.DrawInactiveMagnifier;
var
  W, H: Integer;
  
  procedure DrawAimIcon(Canvas: TCanvas);
  var
    X, Y, A: Integer;
  begin
    X := W div 2;
    Y := H div 2;

    A := 10;
    Canvas.Pen.Color := clBlack;
    Canvas.Brush.Color := clWhite;
    Canvas.Rectangle(X - A, Y - A, X + A + 1, Y + A + 1);
    A := 5;
    Canvas.Brush.Color := clWhite;
    Canvas.Rectangle(X - A, Y - A, X + A + 1, Y + A + 1);

    A := 25;
    Canvas.Pen.Color := clWhite;
    Canvas.Pen.Width := 3;
    Canvas.MoveTo(X - A, Y);  Canvas.LineTo(X + A, Y);
    Canvas.MoveTo(X, Y - A);  Canvas.LineTo(X, Y + A);

    A := 25;
    Canvas.Pen.Color := clBlack;
    Canvas.Pen.Width := 1;
    Canvas.MoveTo(X - A, Y);  Canvas.LineTo(X + A, Y);
    Canvas.MoveTo(X, Y - A);  Canvas.LineTo(X, Y + A);
  end;

var
  Bitmap: TBitmap;
begin
  W := MagnifierImage.Width;
  H := MagnifierImage.Height;

  Bitmap := TBitmap.Create;
  try
    Bitmap.Width := W;
    Bitmap.Height := H;
    with Bitmap.Canvas do
    begin
      Brush.Color := clWhite;
      FillRect(Rect(0, 0, W, H));
    end;
    DrawAimIcon(Bitmap.Canvas);
    MagnifierImage.Picture.Bitmap.Assign(Bitmap);
  finally
    Bitmap.Free;
  end;
end;

procedure TMainForm.DrawColorSampleBox;
begin
  ColorSamplePanel.Color := FCurColor;
end;

procedure TMainForm.ShowMousePos;
var
  P: TPoint;
begin
  if not Iscapturing and not FColorPicked then
  begin
    MousePosEdit.Text := '()';
    Exit;
  end;

  P := Mouse.CursorPos;
  MousePosEdit.Text := Format('%d:%d', [P.X, P.Y]);
end;

procedure TMainForm.ShowColorValue;
var
  Color: TColor;
  R, G, B: Integer;
  ColorStr: string;
begin
  if not Iscapturing and not FColorPicked then
  begin
    ColorValueEdit.Text := '(ɫֵ)';
    Exit;
  end;

  GetRGBFromColor(FCurColor, R, G, B);
  Color := GetColorFromRGB(R, G, B);
  ColorStr := '';

  case FColorDispFormat of
    cdfRGBHex:
      ColorStr := Format('%.2x,%.2x,%.2x', [R, G, B]);
    cdfRGBDec:
      ColorStr := Format('%d,%d,%d', [R, G, B]);
    cdfDelphi:
      ColorStr := Format('$%.8x', [Color]);
    cdfVC:
      ColorStr := Format('0x%.6x', [Color]);
    cdfVB:
      ColorStr := Format('&H%.8x&', [Color]);
    cdfHtml:
      ColorStr := Format('#%.2x%.2x%.2x', [R, G, B]);
  end;

  ColorValueEdit.Text := ColorStr;
end;

procedure TMainForm.ShowTipBox(Value: Boolean);
begin
  TipPaintBox.Visible := Value;
end;

function TMainForm.IsCapturing: Boolean;
begin
  Result := FMagnifierDraging or FHotKeyCapturing;
end;

function TMainForm.IsInMagnifier(const Pos: TPoint): Boolean;
var
  P: TPoint;
begin
  P := MagnifierImage.ScreenToClient(Pos);
  Result := PtInRect(MagnifierImage.BoundsRect, P);
end;

procedure TMainForm.Capture(const ScreenPos: TPoint);

  // 浱ǰĻͼ
  procedure GetCapturedImage(ScrCanvas: TCanvas; ScrX1, ScrY1, ScrX2, ScrY2: Integer);
  begin
    FCurBitmap.Width := (ScrX2 - ScrX1 + 1);
    FCurBitmap.Height := (ScrY2 - ScrY1 + 1);
    FCurBitmap.Canvas.CopyRect(
      Rect(0, 0, FCurBitmap.Width, FCurBitmap.Height),
      ScrCanvas, Rect(ScrX1, ScrY1, ScrX2, ScrY2));
  end;

  // ĻŴͼ
  procedure DrawCapturedImage(ScrCanvas, DestCanvas: TCanvas;
    ScrX1, ScrY1, ScrX2, ScrY2: Integer);
  var
    R: TRect;
  begin
    R := Rect(0, 0, (ScrX2 - ScrX1) * FZoomRate, (ScrY2 - ScrY1) * FZoomRate);
    DestCanvas.CopyRect(R, ScrCanvas, Rect(ScrX1, ScrY1, ScrX2, ScrY2));
  end;

  // ׼ָʾ
  procedure DrawAimIndicator(ScrCanvas, DestCanvas: TCanvas;
    ScrX1, ScrY1, ScrX2, ScrY2: Integer);
  const
    CrossSize = 20;
  var
    X, Y, A: Integer;
    R: TRect;
    ScrColor: TColor;
  begin
    X := ScreenPos.X - ScrX1;
    Y := ScreenPos.Y - ScrY1;
    R := Rect(X * FZoomRate, Y * FZoomRate, (X + 1)* FZoomRate, (Y + 1) * FZoomRate);
    with DestCanvas do
    begin
      ScrColor := ScrCanvas.Pixels[ScreenPos.X, ScreenPos.Y];
      Pen.Mode := pmXor;
      Pen.Color := Iif(ScrColor = $00808080, ScrColor, clWhite);
      if FZoomRate < 5 then
      begin
        MoveTo(X * FZoomRate - CrossSize, Y * FZoomRate);
        LineTo(X * FZoomRate + CrossSize, Y * FZoomRate);
        MoveTo(X * FZoomRate, Y * FZoomRate - CrossSize);
        LineTo(X * FZoomRate, Y * FZoomRate + CrossSize);
      end else
      begin
        Brush.Style := bsClear;
        Rectangle(R);
        A := FZoomRate div 2;
        MoveTo(R.Left - 1, R.Top + A);
        LineTo(R.Left - 1 - CrossSize, R.Top + A);
        MoveTo(R.Right, R.Top + A);
        LineTo(R.Right + CrossSize, R.Top + A);
        MoveTo(R.Left + A, R.Top - 1);
        LineTo(R.Left + A, R.Top - 1 - CrossSize);
        MoveTo(R.Left + A, R.Bottom);
        LineTo(R.Left + A, R.Bottom + CrossSize);
      end;
    end;
  end;

  //  ScreenPos Ļͼ񣬲Ƶ
  procedure CaptureScreen;
  var
    MagBitmap: TBitmap;
    ScrCanvas: TCanvas;
    W, H, CapW, CapH: Integer;
    X1, Y1, X2, Y2: Integer;
  begin
    W := MagnifierImage.Width;
    H := MagnifierImage.Height;

    MagBitmap := TBitmap.Create;
    ScrCanvas := TCanvas.Create;
    try
      MagBitmap.Width := W;
      MagBitmap.Height := H;
      MagBitmap.Canvas.Brush.Color := clWhite;
      MagBitmap.Canvas.FillRect(Rect(0, 0, W, H));

      ScrCanvas.Handle := GetDC(0);

      FZoomRate := Max(1, FZoomRate);
      CapW := Ceil(W div FZoomRate);
      CapH := Ceil(H div FZoomRate);

      // Ļ
      X1 := ScreenPos.X - CapW div 2;
      Y1 := ScreenPos.Y - CapH div 2;
      X1 := EnsureRange(X1, 0, Screen.Width - CapW);
      Y1 := EnsureRange(Y1, 0, Screen.Height - CapH);
      X2 := X1 + CapW;
      Y2 := Y1 + CapH;

      // 浱ǰɫֵ
      FCurColor := ScrCanvas.Pixels[ScreenPos.X, ScreenPos.Y];
      // 浱ǰȡɫλ
      FCurScreenPos := ScreenPos;
      // 浱ǰĻͼ
      GetCapturedImage(ScrCanvas, X1, Y1, X2, Y2);

      // ĻŴͼ
      DrawCapturedImage(ScrCanvas, MagBitmap.Canvas, X1, Y1, X2, Y2);
      // ׼ָʾ
      DrawAimIndicator(ScrCanvas, MagBitmap.Canvas, X1, Y1, X2, Y2);

      // Ƶû
      MagnifierImage.Picture.Bitmap.Assign(MagBitmap);
    finally
      ReleaseDC(0, ScrCanvas.Handle);
      ScrCanvas.Free;
      MagBitmap.Free;
    end;
  end;

begin
  if IsInMagnifier(ScreenPos) then Exit;

  CaptureScreen;
  DrawColorSampleBox;
  ShowMousePos;
  ShowColorValue;
  ShowTipBox(False);
  FColorPicked := True;
end;

procedure TMainForm.MoveByKeyboard(Key: Word; Shift: TShiftState);
var
  Step: Integer;
begin
  Step := 1;
  if ssShift in Shift then Step := 10;

  case Key of
    VK_LEFT:  Dec(FCurScreenPos.X, Step);
    VK_RIGHT: Inc(FCurScreenPos.X, Step);
    VK_UP:    Dec(FCurScreenPos.Y, Step);
    VK_DOWN:  Inc(FCurScreenPos.Y, Step);
  end;
  FCurScreenPos.X := EnsureRange(FCurScreenPos.X, 0, Screen.Width - 1);
  FCurScreenPos.Y := EnsureRange(FCurScreenPos.Y, 0, Screen.Height - 1);

  Mouse.CursorPos := FCurScreenPos;
  Capture(FCurScreenPos);
end;

procedure TMainForm.CopyColorValue;
begin
  if FColorPicked then
  begin
    ColorValueEdit.SelectAll;
    ClipBoard.AsText := ColorValueEdit.Text;
  end;
end;

procedure TMainForm.CopyBitmap;
var
  Format: Word;
  Data: Cardinal;
  Palette: HPALETTE;
begin
  FCurBitmap.SaveToClipBoardFormat(Format, Data, Palette);
  ClipBoard.SetAsHandle(Format, Data);
end;

procedure TMainForm.MagnifierImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
  begin
    Reset;
    FMagnifierDraging := True;
  end;
end;

procedure TMainForm.MagnifierImageMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FMagnifierDraging := False;
end;

procedure TMainForm.MagnifierImageMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  if FMagnifierDraging then
  begin
    Capture(Mouse.CursorPos);
  end;
end;

procedure TMainForm.ColorFmtArrowButtonClick(Sender: TObject);
var
  P: TPoint;
begin
  P := ColorFmtArrowButton.ClientToScreen(Point(ColorFmtArrowButton.Width, 0));
  ColorFmtPopupMenu.Popup(P.X, P.Y);
end;

procedure TMainForm.MenuPanelMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  P: TPoint;
begin
  if MenuPanel.Tag = 1 then
  begin
    MenuPanel.Tag := 0;
    Exit;
  end;

  if MenuPanel.BevelInner = bvRaised then
  begin
    MenuPanel.BevelInner := bvNone;
    P := MenuPanel.ClientToScreen(Point(0, 0));
    if Screen.Height - P.Y > 150 then
      P := MenuPanel.ClientToScreen(Point(0, MenuPanel.Height)); 
    MainPopupMenu.Popup(P.X, P.Y);
    if PtInRect(Rect(0, 0, MenuPanel.Width, MenuPanel.Height),
      MenuPanel.ScreenToClient(Mouse.CursorPos)) then
      MenuPanel.Tag := 1;
  end;
  MenuPanel.BevelInner := bvRaised;
end;

procedure TMainForm.ColorFmtPopItemClick(Sender: TObject);
begin
  FColorDispFormat := TColorDispFormat((Sender as TMenuItem).Tag);
  ShowColorValue;
end;

procedure TMainForm.HotKeySpyHotKeys0HotKey(Sender: TObject);
begin
  // F5
  MainPopStartCapItemClick(nil);
end;

procedure TMainForm.HotKeySpyHotKeys1HotKey(Sender: TObject);
begin
  // F6
  MainPopStopCapItemClick(nil);
end;

procedure TMainForm.HotKeySpyHotKeys2HotKey(Sender: TObject);
begin
  // F8
  MainPopCopyImageItemClick(nil);
end;

procedure TMainForm.CaptureTimerTimer(Sender: TObject);
begin
  Capture(Mouse.CursorPos);
end;

procedure TMainForm.ColorValueEditClick(Sender: TObject);
begin
  CopyColorValue;
end;

procedure TMainForm.ColorValueEditDblClick(Sender: TObject);
begin
  ColorValueEdit.SelLength := 0;
  ColorFmtArrowButtonClick(nil);
end;

procedure TMainForm.ZoomRateButtonClick(Sender: TObject);
begin
  FZoomRate := (Sender as TComponent).Tag;
  (Sender as TSpeedButton).Down := True;
  if IsCapturing or FColorPicked then
    Capture(FCurScreenPos);
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if IsCapturing or FColorPicked then
  begin
    if Key in [VK_LEFT, VK_RIGHT, VK_UP, VK_DOWN] then
      MoveByKeyboard(Key, Shift);
  end;
end;

procedure TMainForm.MainPopStartCapItemClick(Sender: TObject);
begin
  if not FHotKeyCapturing then
  begin
    Reset;
    FHotKeyCapturing := True;
    CaptureTimer.Enabled := True;
  end;
end;

procedure TMainForm.MainPopStopCapItemClick(Sender: TObject);
begin
  if FHotKeyCapturing then
  begin
    FHotKeyCapturing := False;
    CaptureTimer.Enabled := False;
  end;
end;

procedure TMainForm.MainPopCopyImageItemClick(Sender: TObject);
begin
  CopyBitmap;
end;

procedure TMainForm.MainPopCopyColorItemClick(Sender: TObject);
begin
  CopyColorValue;
end;

procedure TMainForm.MainPopHelpItemClick(Sender: TObject);
var
  S: string;
begin
  S := ExtractFilePath(Application.ExeName) + 'Readme.txt';
  ExecuteFile(S, '', '', SW_SHOW);
end;

procedure TMainForm.MainPopExitItemClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.TipPaintBoxPaint(Sender: TObject);
var
  Canvas: TCanvas;
  W, H: Integer;
  Icon: TIcon;
begin
  W := TipPaintBox.Width;
  H := TipPaintBox.Height;

  Canvas := TipPaintBox.Canvas;
  Canvas.Pen.Color := clBlack;
  Canvas.Brush.Color := $00E1FFFF;
  Canvas.Rectangle(Rect(0, 0, W, H));

  Icon := TIcon.Create;
  try
    ImageList.GetIcon(0, Icon);
    Canvas.Draw(3, 3, Icon);
  finally
    Icon.Free;
  end;

  Canvas.Font.Assign(Self.Font);
  Canvas.Brush.Style := bsClear;
  Canvas.TextOut(20, 4, '밴סҷ');
end;

procedure TMainForm.TipPaintBoxMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  ShowTipBox(False);
end;

procedure TMainForm.MainPopupMenuPopup(Sender: TObject);
begin
  MainPopStartCapItem.Enabled := not FHotKeyCapturing;
  MainPopStopCapItem.Enabled := FHotKeyCapturing;
  MainPopCopyImageItem.Enabled := not FCurBitmap.Empty;
  MainPopCopyColorItem.Enabled := FColorPicked;
end;

end.
